var Joystick = function (lra, uda, lrn, udn) {
    this.lra = lra; // left right axes
    this.uda = uda; // up down axes
    this.lrn = lrn; // left right neutral value
    this.udn = udn; // up down neutral value
};
var View = {
    viewer: {
        buttons: [],
        axes: [],
        mid: [],
        joysticks: [],
        axRadius: 20,
        canvas: null,
        ctx: null
    },
    initView: function (gamepad) {
        if(!this.viewer.canvas){
            this.viewer.canvas = document.getElementById("cvAxis");
            this.viewer.ctx = this.viewer.canvas.getContext("2d");
        }
        let bv = $("#dvButtonView");
        for (let b = 0; b < gamepad.buttons.length; b++) {
            let btn = $("<button>").html('' + (b + 1))
                .addClass("btnNorm")
                .css("left", (b * 70) + 'px');
            this.viewer.buttons.push(btn);
            bv.append(btn);
        }
        let diam = this.viewer.axRadius * 2;
        this.viewer.canvas.width = 2 * (diam + 20);
        this.viewer.canvas.height = diam + 2;
        for (let a = 0; a < 2; a++) {
            let ax = { x: a * diam + a * 20 + diam, y: this.viewer.axRadius + 1 };
            this.viewer.joysticks.push(ax);
        }
        for (let a = 0; a < gamepad.axes.length; a++) {
            let lbl = $("<label>").html("0.000").addClass("lbl50")
                .css("left", (a * 70) + 'px');
            this.viewer.axes.push(lbl);
            this.viewer.mid.push(gamepad.axes[a]); // store neutral position values
            bv.append(lbl);
        }
    },
    refreshButtons: function(gpad) {
        for (let b = 0; b < gpad.buttons.length; b++) {
            let val = gpad.buttons[b];
            let pressed = (val === 1.0);
            if (typeof (val) === "object") {
                pressed = val.pressed;
            }
            if (pressed) {
                this.viewer.buttons[b].removeClass("btnNorm").addClass("btnPress");
            } else {
                this.viewer.buttons[b].removeClass("btnPress").addClass("btnNorm");
            }
        }
    },
    refreshAxesValues: function(gpad){
        for (let a = 0; a < gpad.axes.length; a++) {
            this.viewer.axes[a].html(gpad.axes[a].toFixed(4));
            if (gpad.axes[a] - this.viewer.mid[a] < 0) {
                this.viewer.axes[a].removeClass("lbl50").addClass("lblLeft");
            } else {
                if (gpad.axes[a] - this.viewer.mid[a] > 0) {
                    this.viewer.axes[a].removeClass("lbl50").addClass("lblRight");
                } else {
                    this.viewer.axes[a].removeClass("lblLeft lblRight")
                        .addClass("lbl50");
                }
            }
        }
    },
    drawJoysticks: function(gpad, joysticks){
        this.viewer.ctx.clearRect(0, 0, this.viewer.canvas.width,
            this.viewer.canvas.height);
        for (let a = 0; a < 2; a++) {
            let lr = joysticks[a].lra;
            let ud = joysticks[a].uda;
            this.drawAxes(this.viewer.joysticks[a],
                (gpad.axes[lr] === joysticks[a].lrn) ? 0 : gpad.axes[lr],
                (gpad.axes[ud] === joysticks[a].udn) ? 0 : gpad.axes[ud]);
        }
    },
    drawAxes: function(pos, valX, valY){
        let x = pos.x + Math.round(valX * this.viewer.axRadius +
            0.49 * Math.sign(valX));
        let y = pos.y + Math.round(valY * this.viewer.axRadius +
            0.49 * Math.sign(valY));
        this.viewer.ctx.beginPath();
        this.viewer.ctx.arc(pos.x, pos.y, this.viewer.axRadius, 0, Math.PI * 2);
        this.viewer.ctx.stroke();
        this.viewer.ctx.closePath();
        if (valX !== 0 || valY !== 0) {
            this.viewer.ctx.beginPath();
            this.viewer.ctx.moveTo(pos.x, pos.y);
            this.viewer.ctx.lineTo(x, y);
            this.viewer.ctx.stroke();
            this.viewer.ctx.closePath();
        }    
    }
};
var Gamepad = {
    gamepads: {},
    joysticks: [],
    viewShown: false,
    gamepadEvent: function (e, connected) {
        if (connected) {
            this.gamepads[e.gamepad.index] = e.gamepad;
            if (Object.keys(this.gamepads).length === 1) {
                // only displaying status for first connected gamepad
                this._addJoysticks(e.gamepad);
                if(!this.viewShown){
                    View.initView(e.gamepad);
                    this.viewShown = true;
                }
            }
        } else {
            delete this.gamepads[e.gamepad.index];
            this.joysticks = [];
        }
    },
    _addJoysticks: function (gamepad) {
        this.joysticks.push(new Joystick(0, 1, gamepad.axes[0],
            gamepad.axes[1]));
        this.joysticks.push(new Joystick(2, 5, gamepad.axes[2],
            gamepad.axes[5]));
    },
    _scanGamepads: function() {
        let gpads = navigator.getGamepads();
        for (let i = 0; i < gpads.length; i++) {
            if (gpads[i]) {
                this.gamepads[gpads[i].index] = gpads[i];
            }
        }    
    },
    getGamepadInput: function() {
        this._scanGamepads();
        let rVals = [];
        for (let g in this.gamepads) {
            let gpad = this.gamepads[g];
            for (let a = 0; a < 2; a++) {
                let lr = this.joysticks[a].lra;
                let ud = this.joysticks[a].uda;
                let dx = (gpad.axes[lr] === this.joysticks[a].lrn) ? 0 : gpad.axes[lr];
                let dy = (gpad.axes[ud] === this.joysticks[a].udn) ? 0 : gpad.axes[ud];
                rVals.push({x: dx, y: dy});
            }
        }
        return rVals;
    },
    updateView: function(){
            for (let g in this.gamepads) {
                let gpad = this.gamepads[g];
                View.refreshButtons(gpad);
                View.refreshAxesValues(gpad);
                View.drawJoysticks(gpad, this.joysticks);
                break; // exit after first ? only
            }
    }
};